@@ -27,10 +27,8 @@
 ==============================================================================*/
 
 /*
-
 	Written by Stefan Tibus <Stefan.Tibus@ThePentagon.com>
 	Improvements by Andrew Zabolotny <bit@eltech.ru>
-
 */
 
 #ifdef HAVE_CONFIG_H
@@ -39,13 +37,11 @@
 
 #ifdef DRV_OS2
 
+#undef VERSION /* stop conflict with os2medef.h */
 #define INCL_DOS
 #define INCL_OS2MM
 #include <os2.h>
-/* Prevent a warning: PPFN redefined */
-#define PPFN _PPFN
 #include <os2me.h>
-#undef PPFN
 
 #include <stdlib.h>
 #include <string.h>
@@ -61,7 +57,7 @@ static ULONG DeviceIndex = 0;
 static ULONG BufferSize = (ULONG)-1;
 static ULONG DeviceID = 0;
 static PVOID AudioBuffer = NULL;
-static TID ThreadID = NULLHANDLE;
+static int ThreadID = -1;
 static BOOL FinishPlayback;		/* flag for update thread to die */
 static HEV Play = NULLHANDLE;	/* posted on PlayStart event */
 static HEV Update = NULLHANDLE;	/* timer event semaphore */
@@ -83,8 +79,8 @@ void OS2_UpdateBufferThread(void *dummy)
 	DosSetPriority(PRTYS_THREAD, PRTYC_TIMECRITICAL, PRTYD_MAXIMUM - 1, 0);
 
 	while (!FinishPlayback) {
+		static ULONG NextBuffer = 0;  /* next fragment to be filled */
 		ULONG count;
-static	ULONG NextBuffer = 0;	/* next fragment to be filled */
 
 		/* wait for play enable */
 		DosWaitEventSem(Play, SEM_INDEFINITE_WAIT);
@@ -113,7 +109,7 @@ static	ULONG NextBuffer = 0;	/* next fragment to be filled */
 		}
 	}
 	/* Tell main thread we're done */
-	ThreadID = 0;
+	ThreadID = -1;
 }
 
 static void OS2_CommandLine(const CHAR *cmdline)
@@ -121,13 +117,13 @@ static void OS2_CommandLine(const CHAR *cmdline)
 	char *ptr;
 	int buf;
 
-	if ((ptr = MD_GetAtom("buffer", cmdline, 0))) {
+	if ((ptr = MD_GetAtom("buffer", cmdline, 0)) != NULL) {
 		buf = atoi(ptr);
 		if (buf >= 12 && buf <= 16)
 			BufferSize = 1 << buf;
 		MikMod_free(ptr);
 	}
-	if ((ptr = MD_GetAtom("device", cmdline, 0))) {
+	if ((ptr = MD_GetAtom("device", cmdline, 0)) != NULL) {
 		buf = atoi(ptr);
 		if (buf >= 0 && buf <= 8)
 			DeviceIndex = buf;
@@ -166,8 +162,8 @@ static int OS2_Init(void)
 	if (VC_Init())
 		return 1;
 
+	ThreadID = -1;
 	DeviceID = 0;
-	ThreadID = 0;
 	Timer = NULLHANDLE;
 	Update = NULLHANDLE;
 	Play = NULLHANDLE;
@@ -218,7 +214,7 @@ static int OS2_Init(void)
 	memset(&mciOpenParms, 0, sizeof(mciOpenParms));
 	mciOpenParms.pszDeviceType = (PSZ) MAKEULONG(MCI_DEVTYPE_WAVEFORM_AUDIO,
 												 (USHORT) DeviceIndex);
-	mciOpenParms.pszElementName = (PSZ) & PlayList;
+	mciOpenParms.pszElementName = (PSZ) & PlayList[0];
 
 	/* open WAVEAUDIO-device */
 	if (mciSendCommand(0, MCI_OPEN,
@@ -262,7 +258,7 @@ static int OS2_Init(void)
 	/* Create thread for buffer updates */
 	FinishPlayback = FALSE;
 	ThreadID = _beginthread(OS2_UpdateBufferThread, NULL, 0x4000, NULL);
-	if (!ThreadID) {
+	if (ThreadID == -1) {
 		_mm_errno = MMERR_OS2_THREAD;
 		return 1;
 	}
@@ -275,7 +271,7 @@ static void OS2_Exit(void)
 	MCI_GENERIC_PARMS mciGenericParms;
 
 	FinishPlayback = TRUE;
-	while (ThreadID) {
+	while (ThreadID != -1) {
 		DosPostEventSem(Play);
 		DosPostEventSem(Update);
 		DosSleep(1);
@@ -292,13 +288,13 @@ static void OS2_Exit(void)
 		DosCloseEventSem(Play);
 		Play = NULLHANDLE;
 	}
-	VC_Exit();
 	if (DeviceID) {
 		mciGenericParms.hwndCallback = (HWND) NULL;
 		mciSendCommand(DeviceID, MCI_CLOSE, MCI_WAIT,
 					   (PVOID) & mciGenericParms, 0);
 		DeviceID = 0;
 	}
+	VC_Exit();
 	MikMod_free(AudioBuffer);
 	AudioBuffer = NULL;
 }
