@@ -69,8 +69,6 @@ extern int fseek(FILE *, long, int);
 extern size_t fwrite(const void *, size_t, size_t, FILE *);
 #endif
 
-#define COPY_BUFSIZE  1024
-
 /* some prototypes */
 static BOOL _mm_MemReader_Eof(MREADER* reader);
 static BOOL _mm_MemReader_Read(MREADER* reader,void* ptr,size_t size);
@@ -78,8 +76,6 @@ static int _mm_MemReader_Get(MREADER* reader);
 static int _mm_MemReader_Seek(MREADER* reader,long offset,int whence);
 static long _mm_MemReader_Tell(MREADER* reader);
 
-/*static long _mm_iobase = 0, temp_iobase = 0;*/
-
 FILE* _mm_fopen(const CHAR* fname, const CHAR* attrib)
 {
 	FILE *fp;
@@ -154,7 +150,7 @@ static long _mm_FileReader_Tell(MREADER* reader)
 
 MREADER *_mm_new_file_reader(FILE* fp)
 {
-	MFILEREADER* reader=(MFILEREADER*)MikMod_malloc(sizeof(MFILEREADER));
+	MFILEREADER* reader=(MFILEREADER*)MikMod_calloc(1,sizeof(MFILEREADER));
 	if (reader) {
 		reader->core.Eof =&_mm_FileReader_Eof;
 		reader->core.Read=&_mm_FileReader_Read;
@@ -200,7 +196,7 @@ static int _mm_FileWriter_Put(MWRITER* writer,int value)
 
 MWRITER *_mm_new_file_writer(FILE* fp)
 {
-	MFILEWRITER* writer=(MFILEWRITER*)MikMod_malloc(sizeof(MFILEWRITER));
+	MFILEWRITER* writer=(MFILEWRITER*)MikMod_calloc(1,sizeof(MFILEWRITER));
 	if (writer) {
 		writer->core.Seek =&_mm_FileWriter_Seek;
 		writer->core.Tell =&_mm_FileWriter_Tell;
@@ -232,7 +228,7 @@ void _mm_delete_mem_reader(MREADER* reader)
 
 MREADER *_mm_new_mem_reader(const void *buffer, long len)
 {
-	MMEMREADER* reader=(MMEMREADER*)MikMod_malloc(sizeof(MMEMREADER));
+	MMEMREADER* reader=(MMEMREADER*)MikMod_calloc(1,sizeof(MMEMREADER));
 	if (reader)
 	{
 		reader->core.Eof =&_mm_MemReader_Eof;
@@ -320,6 +316,8 @@ static int _mm_MemReader_Seek(MREADER* reader,long offset,int whence)
 	case SEEK_END:
 		mr->pos = mr->len + offset;
 		break;
+	default: /* invalid */
+		return -1;
 	}
 	if (mr->pos < reader->iobase) {
 		mr->pos = mr->core.iobase;
