@@ -37,7 +37,7 @@
 /* Unibuffer chunk size */
 #define BUFPAGE  128
 
-UWORD unioperands[UNI_LAST]={
+const UWORD unioperands[UNI_LAST] = {
 	0, /* not used */
 	1, /* UNI_NOTE */
 	1, /* UNI_INSTRUMENT */
@@ -100,6 +100,27 @@ UWORD unioperands[UNI_LAST]={
 	0, /* UNI_MEDEFFECTF2 */
 	0, /* UNI_MEDEFFECTF3 */
 	2, /* UNI_OKTARP */
+	0, /* not used */
+	1, /* UNI_S3MEFFECTH */
+	1, /* UNI_ITEFFECTH_OLD */
+	1, /* UNI_ITEFFECTU_OLD */
+	1, /* UNI_GDMEFFECT4 */
+	1, /* UNI_GDMEFFECT7 */
+	1, /* UNI_GDMEFFECT14 */
+	2, /* UNI_MEDEFFECT_VIB */
+	0, /* UNI_MEDEFFECT_FD */
+	1, /* UNI_MEDEFFECT_16 */
+	1, /* UNI_MEDEFFECT_18 */
+	1, /* UNI_MEDEFFECT_1E */
+	1, /* UNI_MEDEFFECT_1F */
+	1, /* UNI_FAREFFECT1 */
+	1, /* UNI_FAREFFECT2 */
+	1, /* UNI_FAREFFECT3 */
+	1, /* UNI_FAREFFECT4 */
+	1, /* UNI_FAREFFECT6 */
+	1, /* UNI_FAREFFECTD */
+	1, /* UNI_FAREFFECTE */
+	1, /* UNI_FAREFFECTF */
 };
 
 /* Sparse description of the internal module format
@@ -262,7 +283,7 @@ void UniNewline(void)
 		unibuf[lastp]+=0x20;
 		unipc = unitt+1;
 	} else {
-		if (UniExpand(unitt-unipc)) {
+		if (UniExpand(len)) {
 			/* current and previous row aren't equal... update the pointers */
 			unibuf[unitt] = len;
 			lastp = unitt;
